import os
import pathlib

from .coco_assistant import COCO_Assistant
import coco_assistant

PACKAGE_ROOT = pathlib.Path(coco_assistant.__file__).resolve().parent
#PACKAGE_ROOT = os.getcwd()

with open(os.path.join(PACKAGE_ROOT, 'VERSION')) as version_file:
    __version__ = version_file.read().strip()



import os
import shutil

import tarfile

from coco_assistant import COCO_Assistant

import data_getter

from pycocotools.coco import COCO

import pytest


@pytest.fixture
def get_data():
	if os.path.isdir('./annotations') is False and os.path.isdir('./images') is False:
	    # Download and extract data
	    print("Downloading...")
	    file_id = '1WAFzdtIa56UL4wFVHg2TaBMhtzqRc0F-'
	    destination = 'test.tar.gz'
	    data_getter.download_file_from_google_drive(file_id, destination)
	    # Unzip data
	    print("Extracting")
	    tar = tarfile.open(destination, "r:gz")
	    tar.extractall()
	    tar.close()
	# Set up paths
	img_dir = os.path.join(os.getcwd(), 'images')
	ann_dir = os.path.join(os.getcwd(), 'annotations')
	return [img_dir, ann_dir] 

#@pytest.mark.skip
def test_combine(get_data):
	cas = COCO_Assistant(get_data[0], get_data[1])
	cas.combine()
	comb = COCO(os.path.join(cas.resann_dir, 'combined.json'))
	# Get combined annotation count
	combann = len(comb.anns)
    # Get individual annotation counts
	ann_counts = [len(_cfile.anns) for _cfile in cas.annfiles]
	print(combann)
	print(sum(ann_counts))
    # Clean up
	shutil.rmtree(cas.res_dir)
	assert sum(ann_counts) == combann, "Failure in merging datasets"


#@pytest.mark.skip
def test_cat_removal(get_data):
	cas = COCO_Assistant(get_data[0], get_data[1])

	test_ann = "tiny2.json"
	test_rcats = sorted(['plane', 'ship', 'Large_Vehicle'])

	cas.remove_cat(jc=test_ann, rcats=test_rcats)

	orig = COCO(os.path.join(cas.ann_dir, cas.jc)) 
	rmj = COCO(os.path.join(cas.resrm_dir, cas.jc))

	orig_names = [list(orig.cats.values())[i]['name'] for i in range(len(orig.cats))] 
	rmj_names = [list(rmj.cats.values())[i]['name'] for i in range(len(rmj.cats))]

	diff_names = sorted(list(set(orig_names) - set(rmj_names)))

        # Clean up
	shutil.rmtree(cas.resrm_dir)
	assert diff_names == test_rcats, "Failure in removing following categories: {}".format(test_rcats)



import logging
import os
from random import shuffle

from PIL import Image

from pycocotools.coco import COCO

import tensorflow as tf

from .utils import dataset_util

logging.basicConfig(level=logging.WARNING)

# flags = tf.app.flags
# flags.DEFINE_string('data_dir', '', 'Root directory to raw Microsoft COCO dataset.')
# flags.DEFINE_string('set', 'train', 'Convert training set or validation set')
# flags.DEFINE_string('output_filepath', '', 'Path to output TFRecord')
# flags.DEFINE_bool('shuffle_imgs',True,'whether to shuffle images of coco')
# FLAGS = flags.FLAGS


def load_coco_detection_dataset(imgs_dir, annotations, shuffle_img=True):
    """Load data from dataset by pycocotools. This tools can be download from "http://mscoco.org/dataset/#download"
    Args:
        imgs_dir: directories of coco images
        annotations_filepath: file path of coco annotations file
        shuffle_img: wheter to shuffle images order
    Return:
        coco_data: list of dictionary format information of each image
    """
    coco = annotations
    img_ids = coco.getImgIds()  # totally 82783 images
    cat_ids = coco.getCatIds()  #totally 90 catagories, however, the number of categories is not continuous, \
                               # [0,12,26,29,30,45,66,68,69,71,83] are missing, this is the problem of coco dataset.

    if shuffle_img:
        shuffle(img_ids)

    coco_data = []

    nb_imgs = len(img_ids)
    for index, img_id in enumerate(img_ids):
        if index % 100 == 0:
            print("Readling images: %d / %d " % (index, nb_imgs))
        img_info = {}
        bboxes = []
        labels = []

        img_detail = coco.loadImgs(img_id)[0]
        try:
            pic_height = img_detail['height']
            pic_width = img_detail['width']
        except:
            logging.warning("Image dimension is missing from the image field."
                            " Proceeding to read it manually")
            im = Image.open(os.path.join(imgs_dir, img_detail['file_name']))
            pic_height = im.size[1]
            pic_width = im.size[0]

        ann_ids = coco.getAnnIds(imgIds=img_id, catIds=cat_ids)
        anns = coco.loadAnns(ann_ids)
        for ann in anns:
            bboxes_data = ann['bbox']
            # the format of coco bounding boxs is [Xmin, Ymin, width, height]
            bboxes_data = [bboxes_data[0] / float(pic_width), bboxes_data[1] / float(pic_height),
                           bboxes_data[2] / float(pic_width), bboxes_data[3] / float(pic_height)]
            bboxes.append(bboxes_data)
            labels.append(ann['category_id'])

        img_path = os.path.join(imgs_dir, img_detail['file_name'])
        img_bytes = tf.gfile.FastGFile(img_path, 'rb').read()

        img_info['pixel_data'] = img_bytes
        img_info['height'] = pic_height
        img_info['width'] = pic_width
        img_info['bboxes'] = bboxes
        img_info['labels'] = labels

        coco_data.append(img_info)

    return coco_data


def dict_to_coco_example(img_data):
    """Convert python dictionary formath data of one image to tf.Example proto.
    Args:
        img_data: infomation of one image, inclue bounding box, labels of bounding box,\
            height, width, encoded pixel data.
    Returns:
        example: The converted tf.Example
    """
    bboxes = img_data['bboxes']
    xmin, xmax, ymin, ymax = [], [], [], []
    for bbox in bboxes:
        xmin.append(bbox[0])
        xmax.append(bbox[0] + bbox[2])
        ymin.append(bbox[1])
        ymax.append(bbox[1] + bbox[3])

    example = tf.train.Example(features=tf.train.Features(feature={
        'image/height': dataset_util.int64_feature(img_data['height']),
        'image/width': dataset_util.int64_feature(img_data['width']),
        'image/object/bbox/xmin': dataset_util.float_list_feature(xmin),
        'image/object/bbox/xmax': dataset_util.float_list_feature(xmax),
        'image/object/bbox/ymin': dataset_util.float_list_feature(ymin),
        'image/object/bbox/ymax': dataset_util.float_list_feature(ymax),
        'image/object/class/label': dataset_util.int64_list_feature(img_data['labels']),
        'image/encoded': dataset_util.bytes_feature(img_data['pixel_data']),
        'image/format': dataset_util.bytes_feature('jpeg'.encode('utf-8')),
    }))
    return example


def convert(ann, img_dir, _format):

    dst = os.path.join(os.path.dirname(os.path.dirname(img_dir)),
                       'annotations',
                       os.path.basename(img_dir) + ".tfrecord")

    if _format == "TFRecord":
        # load total coco data
        coco_data = load_coco_detection_dataset(img_dir, ann, shuffle_img=True)
        total_imgs = len(coco_data)
        # write coco data to tf record
        with tf.python_io.TFRecordWriter(dst) as tfrecord_writer:
            for index, img_data in enumerate(coco_data):
                if index % 100 == 0:
                    print("Converting images: %d / %d" % (index, total_imgs))
                example = dict_to_coco_example(img_data)
                tfrecord_writer.write(example.SerializeToString())


if __name__ == "__main__":
    _format = "TFRecord"
    ann = COCO("/home/ashwin/Desktop/keras-retinanet/data/IIAI/annotations/train.json")
    img_dir = "/home/ashwin/Desktop/keras-retinanet/data/IIAI/images/train"
    convert(ann, img_dir, _format)



import json
import logging
import os
import shutil

from pycocotools.coco import COCO

from tqdm import tqdm

from . import coco_converters as converter
from . import coco_stats as stats
from . import coco_visualiser as cocovis

logging.basicConfig(level=logging.ERROR)
logging.getLogger().setLevel(logging.WARNING)
logging.getLogger('parso.python.diff').disabled = True

"""
Expected Directory Structure

.
├── images
│   ├── train
│   ├── val
|   ├── test
|
├── annotations
│   ├── train.json
│   ├── val.json
│   ├── test.json


"""


class COCO_Assistant():
    def __init__(self, img_dir=None, ann_dir=None):
        """
        :param img_dir (str): path to images folder.
        :param ann_dir (str): path to annotations folder.
        """
        self.img_dir = img_dir
        self.ann_dir = ann_dir
        assert os.path.dirname(ann_dir) == os.path.dirname(img_dir)
        self.res_dir = os.path.join(os.path.dirname(ann_dir), 'results')

        # Create Results Directory
        if os.path.exists(self.res_dir) is False:
            os.mkdir(self.res_dir)

        self.imgfolders = [i for i in os.listdir(self.img_dir) if os.path.isdir(os.path.join(self.img_dir, i)) is True]
        self.jsonfiles = sorted([j for j in os.listdir(ann_dir) if j[-5:] == ".json"])
        self.names = [n[:-5] for n in self.jsonfiles]

        # Note: Add check for confirming these folders only contain .jpg and .json respectively
        logging.debug("Number of image folders = {}".format(len(self.imgfolders)))
        logging.debug("Number of annotation files = {}".format(len(self.jsonfiles)))

        assert len(self.jsonfiles) >= 1, "Annotation files not passed"
        assert len(self.imgfolders) >= 1, "Image folders not passed"

        self.annfiles = [COCO(os.path.join(ann_dir, i)) for i in self.jsonfiles]
        self.anndict = dict(zip(self.jsonfiles, self.annfiles))

    def combine(self):
        """
        Function for combining multiple coco datasets
        """

        self.resim_dir = os.path.join(self.res_dir, 'combination', 'images')
        self.resann_dir = os.path.join(self.res_dir, 'combination', 'annotations')

        # Create directories for combination results and clear the previous ones
        # The exist_ok is for dealing with combination folder
        # TODO: Can be done better
        if os.path.exists(self.resim_dir) is False:
            os.makedirs(self.resim_dir, exist_ok=True)
        else:
            shutil.rmtree(self.resim_dir)
            os.makedirs(self.resim_dir, exist_ok=True)
        if os.path.exists(self.resann_dir) is False:
            os.makedirs(self.resann_dir, exist_ok=True)
        else:
            shutil.rmtree(self.resann_dir)
            os.makedirs(self.resann_dir, exist_ok=True)

        # Combine images
        print("Merging image dirs")
        im_dirs = [os.path.join(self.img_dir, folder) for folder in self.imgfolders]
        imext = [".png", ".jpg"]

        logging.debug("Combining Images...")

        for imdir in tqdm(im_dirs):
            ims = [i for i in os.listdir(imdir) if i[-4:].lower() in imext]
            for im in ims:
                shutil.copyfile(os.path.join(imdir, im), os.path.join(self.resim_dir, im))

        # Combine annotations
        cann = {'images': [],
                'annotations': [],
                'info': None,
                'licenses': None,
                'categories': None}

        logging.debug("Combining Annotations...")

        dst_ann = os.path.join(self.resann_dir, 'combined.json')

        print("Merging annotations")
        for j in tqdm(self.jsonfiles):
            with open(os.path.join(self.ann_dir, j)) as a:
                c = json.load(a)

            ind = self.jsonfiles.index(j)
            cocofile = self.annfiles[ind]
            # Check if this is the 1st annotation.
            # If it is, continue else modify current annotation
            if ind == 0:
                cann['images'] = cann['images'] + c['images']
                cann['annotations'] = cann['annotations'] + c['annotations']
                if 'info' in list(c.keys()):
                    cann['info'] = c['info']
                if 'licenses' in list(c.keys()):
                    cann['licenses'] = c['licenses']
                cann['categories'] = c['categories']

                last_imid = cann['images'][-1]['id']
                last_annid = cann['annotations'][-1]['id']

                logging.debug("String Ids detected. Converting to int")

                # If last imid or last_annid is a str, convert it to int
                if type(last_imid) == str:
                    id_dict = {}
                    # Change image id in images field
                    for i, im in enumerate(cann['images']):
                       id_dict[im['id']] = i
                       im['id'] = i

                if type(last_annid) == str:
                    # Change annotation id & image id in annotations field
                    for i, im in enumerate(cann['annotations']):
                        im['id'] = i
                        if type(last_imid) == str:
                            im['image_id'] = id_dict[im['image_id']]

                last_imid = cann['images'][-1]['id']
                last_annid = cann['annotations'][-1]['id']

            else:
                new_imids = [(last_imid + i + 1) for i in sorted(list(cocofile.imgs.keys()))]
                new_annids = [(last_annid + i + 1) for i in sorted(list(cocofile.anns.keys()))]

                def modify_ids(jf, imids, annids):
                    id_dict = {}
                    for img, newimid in zip(jf['images'], imids):
                        id_dict[img['id']] = newimid
                        img['id'] = newimid
                    for ann, newannid in zip(jf['annotations'], annids):
                        ann['id'] = newannid
                        ann['image_id'] = id_dict[ann['image_id']]
                    return jf

                c = modify_ids(c, new_imids, new_annids)
                cann['images'] = cann['images'] + c['images']
                cann['annotations'] = cann['annotations'] + c['annotations']
                if 'info' in list(c.keys()):
                    cann['info'] = c['info']
                if 'licenses' in list(c.keys()):
                    cann['licenses'] = c['licenses']
                cann['categories'] = c['categories']

                last_imid = cann['images'][-1]['id']
                last_annid = cann['annotations'][-1]['id']

        with open(dst_ann, 'w') as aw:
            json.dump(cann, aw)

    def remove_cat(self, jc=None, rcats=None):
        """
        Function for removing certain categories
        """

        self.resrm_dir = os.path.join(self.res_dir, 'removal')
        if os.path.exists(self.resrm_dir) is False:
            os.makedirs(self.resrm_dir, exist_ok=True)
        else:
            shutil.rmtree(self.resrm_dir)
            os.makedirs(self.resrm_dir, exist_ok=True)

        if jc is None or rcats is None:
            ###########################################################
            print(self.jsonfiles)
            print("Who needs a cat removal?")
            self.jc = input()
            assert self.jc.lower() in [item.lower() for item in self.jsonfiles], "Choice not in json file list"
            ind = self.jsonfiles.index(self.jc.lower())
            ann = self.annfiles[ind]

            print("\nCategories present:")
            cats = [i['name'] for i in ann.cats.values()]
            print(cats)

            self.rcats = []
            print("\nEnter categories you wish to remove:")
            while True:
                x = input()
                if x.lower() in [cat.lower() for cat in cats]:
                    catind = [cat.lower() for cat in cats].index(x.lower())
                    self.rcats.append(cats[catind])
                    print(self.rcats)
                    print("Press n if you're done entering categories, else continue")
                elif x == "n":
                    break
                else:
                    print("Incorrect Entry. Enter either the category to be added or press 'n' to quit.")
            ###############################################################

        else:
            self.jc = jc
            ind = self.jsonfiles.index(self.jc.lower())
            ann = self.annfiles[ind]
            self.rcats = rcats

        print("Removing specified categories...")

        # Gives you a list of category ids of the categories to be removed
        catids_remove = ann.getCatIds(catNms=self.rcats)
        # Gives you a list of ids of annotations that contain those categories
        annids_remove = ann.getAnnIds(catIds=catids_remove)

        # Remove from category list
        cats = ann.loadCats(catids_remove)
        # Remove from annotation list
        anns = ann.loadAnns(annids_remove)

        with open(os.path.join(self.ann_dir, self.jc)) as it:
            x = json.load(it)

        x['categories'] = [i for i in x['categories'] if i not in cats]
        x['annotations'] = [i for i in x['annotations'] if i not in anns]

        with open(os.path.join(self.resrm_dir, self.jc), 'w') as oa:
            json.dump(x, oa)

    def ann_stats(self, stat, show_count=False, arearng=[0, 32, 96, 1e5], save=False):
        """
        Function for displaying statistics.
        """
        if stat == "area":
            stats.pi_area_split(self.annfiles, self.names, areaRng=arearng, save=save)
        elif stat == "cat":
            stats.cat_count(self.annfiles, self.names, show_count=show_count, save=save)

    def converter(self, to="TFRecord"):
        """
        Function for converting annotations to other formats
        """
        print("Choose directory:")
        print(self.imgfolders)

        dir_choice = input()

        assert dir_choice.lower() in [item.lower() for item in self.imgfolders], "Choice not in images folder"
        ind = self.imgfolders.index(dir_choice.lower())
        ann = self.annfiles[ind]
        img_dir = os.path.join(self.img_dir, dir_choice)

        converter.convert(ann, img_dir, _format=to)

    def visualise(self):
        """
        Function for visualising annotations.
        """
        print("Choose directory:")
        print(self.imgfolders)

        dir_choice = input()

        assert dir_choice.lower() in [item.lower() for item in self.imgfolders], "Choice not in images folder"
        ind = self.imgfolders.index(dir_choice.lower())
        ann = self.annfiles[ind]
        img_dir = os.path.join(self.img_dir, dir_choice)
        cocovis.visualise_all(ann, img_dir)


if __name__ == "__main__":
    p = "/home/ashwin/Desktop/Projects/COCO-Assistant"
    img_dir = os.path.join(p, 'images')
    ann_dir = os.path.join(p, 'annotations')

    # TODO: Create tiny dummy datasets and test these functions on them

    cas = COCO_Assistant(img_dir, ann_dir)

    cas.combine()
    #cas.remove_cat()
    #cas.ann_stats(stat="area",arearng=[10,144,512,1e5],save=False)
    #cas.ann_stats(stat="cat", show_count=False, save=False)
    #cas.visualise()



import logging
import os

import matplotlib.pyplot as plt

import pandas as pd

from pycocotools.coco import COCO

import seaborn as sns


logging.basicConfig(level=logging.DEBUG)


def cat_count(anns, names, show_count=False, save=False):

    fig, axes = plt.subplots(1, len(anns), sharey=False)

    # Making axes iterable if only single annotation is present
    if len(anns) == 1:
        axes = [axes]

    # Prepare annotations dataframe
    # This should be done at the start
    for ann, name, ax in zip(anns, names, axes):
        ann_df = pd.DataFrame(ann.anns).transpose()
        if 'category_name' in ann_df.columns:
            chart = sns.countplot(data=ann_df,
                                  x='category_name',
                                  order=ann_df['category_name'].value_counts().index,
                                  palette='Set1',
                                  ax=ax)
        else:
            # Add a new column -> category name
            ann_df['category_name'] = ann_df.apply(lambda row: ann.cats[row.category_id]['name'],axis=1)
            chart = sns.countplot(data=ann_df,
                                  x='category_name',
                                  order=ann_df['category_name'].value_counts().index,
                                  palette='Set1',
                                  ax=ax)

        chart.set_title(name)
        chart.set_xticklabels(chart.get_xticklabels(), rotation=90)

        if show_count is True:
            for p in chart.patches:
                height = p.get_height()
                chart.text(p.get_x() + p.get_width() / 2.,
                           height + 0.9,
                           height,
                           ha="center")

    plt.suptitle('Instances per category', fontsize=14, fontweight='bold')
    plt.tight_layout()

    fig = plt.gcf()
    fig.set_size_inches(11, 11)

    out_dir = os.path.join(os.getcwd(), 'results', 'plots')
    if save is True:
        if os.path.exists(out_dir) is False:
            os.mkdir(out_dir)
        plt.savefig(os.path.join(out_dir, "cat_dist" + ".png"),
                    bbox_inches='tight',
                    pad_inches=0,
                    dpi=plt.gcf().dpi)

    plt.show()


def get_areas(ann):
    obj_areas = []
    for key in ann.anns:
        obj_areas.append(ann.anns[key]['area'])
    return obj_areas


def view_area_dist(ann):
    obj_areas = get_areas(ann)
    import matplotlib.pyplot as plt
    plt.plot(range(len(obj_areas)), obj_areas)
    plt.xlabel("Objects")
    plt.ylabel("Areas")
    plt.title("Area Distribution")
    plt.show()

def get_object_size_split(ann, areaRng):

    obj_areas = get_areas(ann)

    assert areaRng == sorted(areaRng), "Area ranges incorrectly provided"

    small    = len(ann.getAnnIds(areaRng=[(areaRng[0]**2), areaRng[1]**2]))
    medium   = len(ann.getAnnIds(areaRng=[(areaRng[1]**2), areaRng[2]**2]))
    large    = len(ann.getAnnIds(areaRng=[(areaRng[2]**2), areaRng[3]**2]))
    left_out = len(ann.getAnnIds(areaRng=[0**2, (areaRng[0]**2)])) + len(ann.getAnnIds(areaRng=[areaRng[3]**2, (1e5**2)]))

    logging.debug("Number of small objects in set = {}".format(small))
    logging.debug("Number of medium objects in set = {}".format(medium))
    logging.debug("Number of large objects in set = {}".format(large))
    if left_out != 0:
        logging.debug("Number of objects ignored in set = {}".format(left_out))

    logging.debug("Number of objects = {}".format(len(obj_areas)))

    assert len(obj_areas) == small + medium + large + left_out, "Sum of objects in different area ranges != Total number of objects"
    return small, medium, large, left_out


def pi_area_split_single(ann, areaRng):
    
    # Pie chart
    small, medium, large, left_out = get_object_size_split(ann, areaRng)

    if left_out != 0:
    	sizes = [small, large, left_out, medium]
    	labels = 'Small', 'Large', 'Ignored', 'Medium'
    	colors = ['#ff9999', '#66b3ff', '#99ff99', '#ffcc99']
    else:
    	sizes = [small, large, medium]
    	labels = 'Small', 'Large', 'Medium'
    	colors = ['#ff9999', '#66b3ff', '#ffcc99']


    fig1, ax1 = plt.subplots()
    ax1.pie(sizes, colors=colors, labels=labels, autopct='%1.2f%%', startangle=90)
    #draw circle
    centre_circle = plt.Circle((0, 0), 0.70, fc='white')
    fig = plt.gcf()
    fig.gca().add_artist(centre_circle)
    # Equal aspect ratio ensures that pie is drawn as a circle
    ax1.axis('equal')  
    plt.title('Object Size Distribution', fontsize=14, fontweight='bold', pad=20)
    plt.tight_layout()
    plt.show()



def pi_area_split(anns, names, areaRng, save=False):

    stuff = []

    for ann in anns:
        small, medium, large, left_out = get_object_size_split(ann, areaRng)

        if left_out != 0:
            sizes = [small, large, left_out, medium]
            labels = 'Small', 'Large', 'Ignored', 'Medium'
            colors = ['#ff9999', '#66b3ff', '#99ff99', '#ffcc99']
        else:
            sizes = [small, large, medium]
            labels = 'Small', 'Large', 'Medium'
            colors = ['#ff9999', '#66b3ff', '#ffcc99']

        stuff.append([sizes, labels, colors])

    fig, axs = plt.subplots(1, len(anns), figsize=(11, 11))

    for s, name, ax in zip(stuff, names, axs.flat):
        ax.clear()
        ax.pie(s[0], labels=s[1], colors=s[2], autopct='%1.2f%%', startangle=90)
        #draw circle
        #centre_circle = plt.Circle((0, 0), 0.70, fc='white')
        #fig = plt.gcf()
        #fig.gca().add_artist(centre_circle)
        # Equal aspect ratio ensures that pie is drawn as a circle
        #ax.axis('equal')
        #from matplotlib import rcParams
        #rcParams['axes.titlepad'] = 100 
        ax.set_title(name)

    #plt.title('Object Size Distribution', pad=20)
    fig.suptitle("Object Size Distribution", fontsize=14, fontweight='bold')
    #plt.tight_layout()

    #fig = plt.gcf()
    #fig.set_size_inches(11,11)

    out_dir = os.path.join(os.getcwd(), 'results','plots')

    if save is True:
        if os.path.exists(out_dir) is False:
            os.mkdir(out_dir)
        plt.savefig(os.path.join(out_dir, "area_dist" + ".png"), dpi=fig.dpi)
    plt.show()


if __name__ == "__main__":
    # Get Annotations Dir and Image folder
    folder1 = "test1"
    annFile1 = os.path.join(os.getcwd(), 'annotations', "{}.json".format(folder1))
    ann1 = COCO(annFile1)

    folder2 = "test2"
    annFile2 = os.path.join(os.getcwd(), 'annotations', "{}.json".format(folder2))
    ann2 = COCO(annFile2)

    folder3 = "test3"
    annFile3 = os.path.join(os.getcwd(), 'annotations', "{}.json".format(folder3))
    ann3 = COCO(annFile3)

    #cat_count(ann, folder, show_count=True)
    #pi_area_split(ann, areaRng=[0, 144, 512, 1e5])
    pi_area_split_multi([ann1, ann2, ann3], folders=[folder1, folder2, folder3], areaRng=[10, 144, 512, 1e5])
    #view_area_dist(ann)



import os

import matplotlib.patches
import matplotlib.pyplot as plt
import matplotlib.widgets

import mpl_toolkits.axes_grid1

from pycocotools.coco import COCO

import skimage.io as io


# Reference: https://stackoverflow.com/questions/41545664/view-3-dimensional-numpy-array-in-matplotlib-and-taking-arguments-from-keyboard/41552601#41552601
# Need to make a minor change in the showAnns function in coco.py which involves passing in a matplotlib axes. Changes are as follows:
# 1. def showAnns(self, anns) -> def showAnns(self,anns, ax=None)
# 2. ax = plt.gca() -> if ax = None: ax = plt.gca()

class ImageSlider(matplotlib.widgets.Slider):

    def __init__(self, ax, label, numpages=10, valinit=0, valfmt='%1d',
                 closedmin=True, closedmax=True,
                 dragging=True, **kwargs):

        self.facecolor = kwargs.get('facecolor', "w")
        self.activecolor = kwargs.pop('activecolor', "b")
        self.fontsize = kwargs.pop('fontsize', 10)
        self.numpages = numpages
        self.fig = ax.figure

        super(ImageSlider, self).__init__(ax, label, 0, numpages,
            valinit=valinit, valfmt=valfmt, **kwargs)

        self.poly.set_visible(False)
        self.vline.set_visible(False)
        self.pageRects = []
        for i in range(numpages):
            facecolor = self.activecolor if i == valinit else self.facecolor
            r = matplotlib.patches.Rectangle((float(i) / numpages, 0), 1. / numpages, 1,
                                transform=ax.transAxes, facecolor=facecolor)
            ax.add_artist(r)
            self.pageRects.append(r)
            ax.text(float(i) / numpages + 0.5 / numpages, 0.5, str(i + 1),
                    ha="center", va="center", transform=ax.transAxes,
                    fontsize=self.fontsize)
        self.valtext.set_visible(False)

        divider = mpl_toolkits.axes_grid1.make_axes_locatable(ax)
        bax = divider.append_axes("right", size="5%", pad=0.05)
        fax = divider.append_axes("right", size="5%", pad=0.05)
        self.button_back = matplotlib.widgets.Button(bax, label='<-',
                        color=self.facecolor, hovercolor=self.activecolor)
        self.button_forward = matplotlib.widgets.Button(fax, label='->',
                        color=self.facecolor, hovercolor=self.activecolor)
        self.button_back.label.set_fontsize(self.fontsize)
        self.button_forward.label.set_fontsize(self.fontsize)
        self.button_back.on_clicked(self.backward)
        self.button_forward.on_clicked(self.forward)
        #connect keys:
        self.fig.canvas.mpl_connect('key_press_event', self.keyevent)

    def _update(self, event):
        super(ImageSlider, self)._update(event)
        i = int(self.val)
        if i >= self.valmax:
            return
        self._colorize(i)

    def _colorize(self, i):
        for j in range(self.numpages):
            self.pageRects[j].set_facecolor(self.facecolor)
        self.pageRects[i].set_facecolor(self.activecolor)

    def forward(self, event):
        current_i = int(self.val)
        i = current_i + 1
        if (i < self.valmin) or (i >= self.valmax):
            return
        self.set_val(i)
        self._colorize(i)

    def backward(self, event):
        current_i = int(self.val)
        i = current_i - 1
        if (i < self.valmin) or (i >= self.valmax):
            return
        self.set_val(i)
        self._colorize(i)

    # define keyevent, left: backwards, right: forwards
    def keyevent(self, event):
        # print event.key
        if event.key == 'right':
            self.forward(event)
        if event.key == 'left':
            self.backward(event)
        self.fig.canvas.draw()


def get_imgid_dict(ann):
    """
    Returns a dictionary with img ids as keys and img filenames as values
    """
    id_fn_dict = {}
    for item in ann.imgs.items():
        id_fn_dict[item[1]['file_name']] = item[0]
    return id_fn_dict


def visualise_all(ann, img_dir):

    # Get List of Images
    imgs = os.listdir(img_dir)

    # Get image id and image filename mapping dict
    id_fn_dict = get_imgid_dict(ann)

    num_pages = len(ann.imgs.keys())

    # Visualise first image

    fig, ax = plt.subplots()
    fig.subplots_adjust(bottom=0.18)

    im = io.imread(os.path.join(img_dir, imgs[0]))
    imgid = id_fn_dict[imgs[0]]
    # Modification for string image ids
    if type(imgid) == str:
        imgid = [imgid]
    annids = ann.getAnnIds(imgIds=imgid, iscrowd=None)
    anns = ann.loadAnns(annids)
    ax.axis('off')
    ax.set_title(imgs[0])
    ax.imshow(im)
    ann.showAnns(anns)

    ax_slider = fig.add_axes([0.05, 0.05, 0.9, 0.04])
    slider = ImageSlider(ax_slider, 'Image', num_pages, activecolor="orange")

    def update(val):
        ax.clear()
        ind = int(slider.val)
        im = io.imread(os.path.join(img_dir, imgs[ind]))
        imid = id_fn_dict[imgs[ind]]
        # Modification for string image ids
        if type(imid) == str:
            imid = [imid]
        img_annids = ann.getAnnIds(imgIds=imid, iscrowd=None)
        img_anns = ann.loadAnns(img_annids)
        ax.axis('off')
        ax.set_title(imgs[ind])
        ax.imshow(im)
        ann.showAnns(img_anns, ax=ax)
    slider.on_changed(update)

    plt.show()


def visualise_single(ann, folder, img_filename):
    assert folder in ['train', 'val', 'test']
    # Get image id and image filename mapping dict
    id_fn_dict = get_imgid_dict(ann)
    img_path = os.path.join(os.getcwd(), "images", folder, img_filename)
    im = io.imread(img_path)
    annids = ann.getAnnIds(imgIds=id_fn_dict[img_filename], iscrowd=None)
    anns = ann.loadAnns(annids)

    # load and display instance annotations
    plt.figure(figsize=(15, 15))
    plt.imshow(im)
    plt.axis('off')
    plt.title(img_filename)
    ann.showAnns(anns)
    plt.show()


if __name__ == "__main__":
    # Get Annotations Dir and Image folder
    folder = "test"
    annFile = os.path.join(os.getcwd(), 'annotations', "iSAID_{}.json".format(folder))
    ann = COCO(annFile)
    # Visualisation Modes
    #visualise_all(ann, folder)
    visualise_single(ann, folder, 'P0009.png')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
import os
from pathlib import Path

from setuptools import find_packages, setup


# Package meta-data.
NAME = 'coco_assistant'
DESCRIPTION = 'Helper for dealing with MS-COCO annotations'
URL = 'https://github.com/ashnair1/COCO-Assistant'
EMAIL = 'ash1995@gmail.com'
AUTHOR = 'Ashwin Nair'
REQUIRES_PYTHON = '>=3.6.0'


# What packages are required for this module to be executed?
def list_reqs(fname='requirements.txt'):
    with open(fname) as f:
        requirements = f.read().splitlines()

    required = []
    dependency_links = []
    # do not add to required lines pointing to git repositories
    EGG_MARK = '#egg='
    for line in requirements:
        if line.startswith('-e git:') or line.startswith('-e git+') or \
                line.startswith('git:') or line.startswith('git+'):
            if EGG_MARK in line:
                package_name = line[line.find(EGG_MARK) + len(EGG_MARK):]
                # Ignore possible subdirectories
                if '&' in package_name:
                    package_name = package_name[0:package_name.find('&')]
                required.append(package_name)
                dependency_links.append(line)
            else:
                print('Dependency to a git repository should have the format:')
                print('git+ssh://git@github.com/xxxxx/xxxxxx#egg=package_name')
        else:
            required.append(line)

    return required, dependency_links


# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the
# Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.md' is present in your MANIFEST.in file!
try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except OSError:
    long_description = DESCRIPTION


# Load the package's __version__.py module as a dictionary.
ROOT_DIR = Path(__file__).resolve().parent
PACKAGE_DIR = ROOT_DIR / NAME
about = {}
with open(PACKAGE_DIR / 'VERSION') as f:
    _version = f.read().strip()
    about['__version__'] = _version

required, dependency_links = list_reqs()

# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    download_url = 'https://github.com/ashnair1/COCO-Assistant/archive/v0.1.0.tar.gz',
    packages=find_packages(exclude=('tests',)),
    package_data={'coco_assistant': ['VERSION']},
    install_requires=required,
    dependency_links=dependency_links,
    extras_require={},
    include_package_data=True,
    license='MIT',
    classifiers=[
        # Trove classifiers
        # Full list: https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
)
